// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "game/Game.hpp"
#include "scene/SceneItem.hpp"

namespace Dawn {
  class Scene final : public std::enable_shared_from_this<Scene> {
    private:
      std::weak_ptr<Game> game;
      const std::function<void(Scene&)> sceneInitializer;
      std::vector<std::shared_ptr<SceneItem>> sceneItems;

    public:
      /**
       * Constructs a scene object.
       * 
       * @param game Game that initialized this scene.
       */
      Scene(
        const std::weak_ptr<Game> game,
        const std::function<void(Scene&)> sceneInitializer
      );

      /**
       * Stages all of the scene items on the scene.
       */
      void stage();

      /**
       * Returns a copy of the game instance that this scene belongs to.
       * 
       * @return Shared pointer to the game instance.
       */
      std::shared_ptr<Game> getGame();

      /**
       * Creates a scene item that belongs to this scene.
       * 
       * @return Pointer to the new scene item.
       */
      std::shared_ptr<SceneItem> createSceneItem();

      /**
       * Returns a list of scene components that match the given type.
       * 
       * @return List of scene components matching the type.
       */
      template<class T>
      std::vector<std::shared_ptr<T>> findComponents() {
        std::vector<std::shared_ptr<T>> components;
        for(auto item : sceneItems) {
          auto component = item->getComponent<T>();
          if(component) components.push_back(component);
        }
        return components;
      }

      /**
       * Destroys the scene object and cleans up all of its children.
       */
      virtual ~Scene();
  };
}